#ifndef _ALLOYWM_H
#define _ALLOYWM_H

/* A few options here. */
#define SHAPE

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#ifdef SHAPE
#include <X11/extensions/shape.h>
#endif

#define APPNAME "alloywm"
#define APPVERS "0.4.1"

/* Default settings */
#define DEF_RESIZEBAR	1
#define DEF_OPAQUE	0
#define DEF_TERM	"xterm"
#define DEF_FONT	"lucidasans-10"
#define DEF_FG		"#000000"
#define DEF_BG		"#d0d0d0"
#define DEF_BC		"#000000"
#define DEF_BC2		"#ffffff"
#define DEF_NEW1	"gtk-panel-menu"
#define DEF_NEW2	"gtk-switch-menu"
#define DEF_BW		1
#define SPACE		1
#define MINSIZE		theight(c)*2
#define CLICKFOCUS	1
#define MOUSEFOCUS	0
#define SLOPPYFOCUS	0
#define STICKINESS	10
#undef DEBUG

#define CYCLE_MOD (Mod1Mask)
#define CYCLE_KEY (XK_Tab)

/* Some coding shorthand */
#define ChildMask	(SubstructureRedirectMask|SubstructureNotifyMask)
#define ButtonMask	(ButtonPressMask|ButtonReleaseMask)
#define MouseMask	(ButtonMask|PointerMotionMask)

/* Quick Note on alloywm Coding Conventions (not in aewm):
   some_procedure() manipulates graphics within the parent
     of the current window (e.g., the titlebar.)
   someProcedure() manipulates graphics within the contents
     of the current window. (to be used later on) */

/* Quickies */
#define lower_client(c) \
	(XLowerWindow(dpy, c->parent))
#define awidget_clicked(widget) \
	(e->button == Button1) && \
	(e->x >= widget.x + opt_bw/2 && \
	 e->x <= widget.x - opt_bw/2 + widget.width && \
	 e->y >= widget.y + opt_bw/2 && \
	 e->y <= widget.y - opt_bw/2 + widget.height)
     
/* Coding shorthand for drawing lines and strings, etc. */
#define draw_line(x1, y1, x2, y2) \
	(XDrawLine(dpy, c->parent, border_gc, \
	x1, y1, x2, y2))
#define draw_string(x, y, string) \
	(XDrawString(dpy, c->parent, string_gc, \
	x, y, string, strlen(c->name)))

#define streq(s1, s2) \
	(strcmp(s1, s2) == 0)
#ifndef DEF_TRANS
#define theight(c) \
	(c->trans ? 0 : iheight + opt_bw)
#endif
#ifdef DEF_TRANS
#define theight(c) \
	(iheight + opt_bw)
#endif
#ifndef DEF_TRANS
#define rheight(c) \
	(c->trans || !DEF_RESIZEBAR ? 0 : (iheight + opt_bw)/2)
#endif
#ifdef DEF_TRANS
#define rheight(c) \
	(!DEF_RESIZEBAR ? 0 : (iheight + opt_bw)/2)
#endif
#define grab(w, mask, curs) \
	(XGrabPointer(dpy, w, False, mask, GrabModeAsync, GrabModeAsync, \
	None, curs, CurrentTime) == GrabSuccess)
#define ungrab() \
	XUngrabServer(dpy); XUngrabPointer(dpy, CurrentTime)
#define setmouse(w, x, y) \
	XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y)
#define gravitate(c) \
	change_gravity(c, 1)
#define ungravitate(c) \
	change_gravity(c, -1)
	
/* Widget structure */
typedef struct AWidget {
	int x;
	int y;
	int width;
	int height;
	int type;
	int state;
} AWidget;

/* Titlebar structure */
typedef struct ATitlebar ATitlebar;
struct ATitlebar {
	char *title;
	int width;
	int height;
	AWidget maximbox, iconfbox, minimbox, closebox;
};

/* Client structure */
typedef struct Client Client;
struct Client {
	Client	*next;
	Window	window;
	Window	parent;
	Window	trans;

	char	*name;

	Colormap	cmap;
	XSizeHints	*size;
	int		ignore_unmap;

	ATitlebar	titlebar;
	
	int		x;
	int		y;
	int		width;
	int		height;
	int		border;

	Bool		shaded;
	Bool		shaped;
};

/* Declarations for global variables in main.c */
extern Display		*dpy;
extern int		screen;
extern Window		root;
extern GC		invert_gc;
extern GC		string_gc;
extern GC		border_gc;
extern GC		threeD_gc;
extern GC		bkfill_gc;
extern XFontStruct	*font;
extern int		iheight;
extern XColor		fg, bg, bc;
extern Client		*head_client;
extern Client		*current;
extern Atom		xa_wm_state;
extern Atom		xa_wm_change_state;
extern Atom		xa_wm_protos;
extern Atom		xa_wm_delete;
extern Atom		xa_wm_cmapwins;
extern Cursor		move_curs;
extern Cursor		resize_curs;
extern char		*opt_title;
extern char		*opt_term;
extern char		*opt_display;
extern char		*opt_font;
extern char		*opt_fg;
extern char		*opt_bg;
extern char		*opt_bc;
extern char		*opt_bc2;
extern char		*opt_new2;
extern char		*opt_new1;
extern int		opt_bw;
#ifdef SHAPE
extern Bool		shape;
extern int		shape_event;
#endif

/* events.c */
void handle_key_event(XKeyEvent *e);
void handle_button_event(XButtonEvent *e);
void handle_client_message(XClientMessageEvent *e);
void handle_colormap_change(XColormapEvent *e);
void handle_configure_request(XConfigureRequestEvent *e);
void handle_enter_event(XCrossingEvent *e);
void handle_expose_event(XExposeEvent *e);
void handle_map_request(XMapRequestEvent *e);
void handle_property_change(XPropertyEvent *e);
void handle_reparent_event(XReparentEvent *e);
#ifdef SHAPE
void handle_shape_change(XShapeEvent *e);
#endif
void handle_unmap_event(XUnmapEvent *e);

/* tooltip.c */
void draw_tooltip(Window win, int x, int y, char *string);
void create_tooltip_window(char *string, int x, int y, int w, int h);
void destroy_tooltip_window();

/* widgets.c */
/* The below macros define types of widgets to be contained in the AWidget
   structure. */
#define AWidget_Empty 1
#define AWidget_Maximise 2
#define AWidget_Iconify 3
#define AWidget_Title 4
#define AWidget_Minimise 5
#define AWidget_Close 6
#define AWidget_Resize 7
#define AWidget_Menu 8

/* Quick macros to define where a widget shall be drawn. */
#define AWidget_Close_X_Position() \
	(c->width - theight(c) + opt_bw/2)
#define AWidget_Minimise_X_Position() \
	(c->width - theight(c)*2 + opt_bw/2)
#define AWidget_Maximise_X_Position() \
	(-1)
#define AWidget_Iconify_X_Position() \
	(theight(c) - opt_bw/2 - opt_bw)

/* The below macros define different states for these widgets to be in. */
#define AState_Normal 0
#define AState_Depressed 1

void position_awidget(Client *c, AWidget *widget);
void draw_line3d(Client *c, int x1, int y1, int x2, int y2);
void draw_inv_line3d(Client *c, int x1, int y1, int x2, int y2);
AWidget make_awidget(int AWidget_Type, int x, int y, int w, int h);
void redraw_awidget(Client *c, AWidget widget);
void redraw_atitlebar(Client *c, ATitlebar titlebar);
void draw_empty_widget(Client *c, int x, int y);
void draw_depressed_empty_widget(Client *c, int x, int y);
void draw_depressed_widget(Client *c, AWidget widget);
void draw_closebox(Client *c, int x, int y);
void draw_minimisebox(Client *c, int x, int y);
void draw_maximisebox(Client *c, int x, int y);
void draw_menubox(Client *c, int x, int y);
void draw_resizebox(Client *c, int x, int y);
void draw_iconifybox(Client *c, int x, int y);
void update_atitlebar(Client *c, ATitlebar *titlebar);

/* pixmap.c */
void draw_xpm(Client *c, XImage *ximage, int x, int y, int w, int h);
XImage *load_xpm(Client *c, char *file);

/* main.c */
int main(int argc, char *argv[]);
void scan_windows();
void setup_display();

/* misc.c */
int handle_xerror(Display *dpy, XErrorEvent *e);
void do_event_loop();
void dump_clients();
void err(char *fmt, ...);
void quit_nicely();
void show_event(XEvent e);
void spawn(char *cmd);
int StashEventTime(XEvent *ev);

/* new.c */
void init_position(Client *c);
void initialize(Client *c, Window w);
void make_new_client(Window w);
void reparent(Client *c);

/* screen.c */
void drag(Client *c);
void draw_outline(Client *c);
void get_mouse_position(int *x, int *y);
void hide(Client *c);
void move(Client *c);
void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2);
void resize(Client *c);
void sweep(Client *c);
void unhide(Client *c);
void rollup(Client *c);
void next(Client *c);

/* client.c */
Client *find_client(Window w, int from_parent);
int send_xmessage(Window w, Atom a, long x);
int wm_state(Client *c);
void change_gravity(Client *c, int multiplier);
void redraw(Client *c);
void maximise(Client *c);
void raise_client(Client *c);
void remove_client(Client *c, int from_cleanup);
void send_config(Client *c);
void send_wm_delete(Client *c);
void set_shape(Client *c);
void set_wm_state(Client *c, int state);

#endif
