/* alloywm - (Somewhat) Minimalist Window Manager for the X Window system
 * Copyright (c) 2002, Kensuke Otake <kensuke@phreaker.net>
 * see README for license and other mindless garbage. */

#include "alloywm.h"
#include <stdlib.h>

Client *find_client(Window w, int from_parent) {
  Client *c;

  if (w && w != root)
    for (c = head_client; c; c = c->next)
      if (w == (from_parent ? c->parent : c->window)) return c;
  return NULL;
}

void set_wm_state(Client *c, int state) {
  long data[2];

  data[0] = (long) state;
  data[1] = None; /* icon window */

  XChangeProperty(dpy, c->window, xa_wm_state, xa_wm_state,
    32, PropModeReplace, (unsigned char *)data, 2);
}

int wm_state(Client *c) {
  Atom real_type;
  int real_format, state;
  unsigned long n, extra;
  unsigned char *data;

  if ((XGetWindowProperty(dpy, c->window, xa_wm_state, 0L, 2L, False,
        AnyPropertyType, &real_type, &real_format, &n,
        &extra, &data) == Success) && n) {
    state = *(int *)data;
    XFree(data);
    return state;
  } else return WithdrawnState;
}

void rollup(Client *c) {
  if (c && !c->shaded) {
    XMoveResizeWindow(dpy, c->parent, c->x, c->y - theight(c),
      c->width, theight(c) - opt_bw);
    c->shaded = True;
  } else if (c && c->shaded) {
    XMoveResizeWindow(dpy, c->parent, 
      c->x, c->y - theight(c), c->width, c->height + theight(c) + rheight(c));
    c->shaded = False;
  }
  redraw(c);
  send_config(c);
}

void maximise(Client *c) {
  int xmax = 0;
  int ymax = 0;  
  
#ifdef DEBUG
  err("maximised %s", c->name);
#endif
  
  if (c && wm_state(c) == IconicState) rollup(c);

  XGrabServer(dpy);
  
  xmax = DisplayWidth(dpy, screen) - opt_bw;
  ymax = DisplayHeight(dpy, screen) - opt_bw;

  c->width = xmax - opt_bw*2;
  c->height = ymax - theight(c) - rheight(c);
  c->x = 0;
  c->y = theight(c);

  XMoveResizeWindow(dpy, c->parent, 0, 0, xmax - opt_bw*2, ymax - opt_bw*2);
  XMoveResizeWindow(dpy, c->window, 0, theight(c), xmax - opt_bw*2, ymax - theight(c) - rheight(c) - opt_bw*2);
  
  send_config(c);
  XUngrabServer(dpy);
}

void raise_client(Client *c) {
  if (c) {
    XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
    XRaiseWindow(dpy, c->parent);
  }
}

void send_config(Client *c) {
  XConfigureEvent ce;

  ce.type = ConfigureNotify;
  ce.event = c->window;
  ce.window = c->window;
  ce.x = c->x;
  ce.y = c->y;
  ce.width = c->width;
  ce.height = c->height;
  ce.border_width = 0;
  ce.above = None;
  ce.override_redirect = 0;

  XSendEvent(dpy, c->window, False, StructureNotifyMask, (XEvent *)&ce);
}

void remove_client(Client *c, int from_cleanup) {
  Client *p;

  XGrabServer(dpy);

  if (from_cleanup) {
    if (wm_state(c) == IconicState) XMapWindow(dpy, c->window);
  } else set_wm_state(c, WithdrawnState);

  ungravitate(c);
  XRemoveFromSaveSet(dpy, c->window);
  XReparentWindow(dpy, c->window, root, c->x, c->y);
  XSetWindowBorderWidth(dpy, c->window, 1);
  XDestroyWindow(dpy, c->parent);

  if (head_client == c) head_client = c->next;
  else for (p = head_client; p && p->next; p = p->next)
    if (p->next == c) p->next = c->next;

  if (c->name) XFree(c->name);
  if (c->size) XFree(c->size);
  current = NULL;
  free(c);

  XSync(dpy, False);
  ungrab();
}

void redraw(Client *c) {
  if (c) {
    /* Shit happens.  So let's keep our title bar and its little
       widgets up to date, alright?  Also, don't waste precious CPU cycles
       on those shit-for-brains transient windows. */
    if (!c->trans) {
      update_atitlebar(c, &c->titlebar);
      redraw_atitlebar(c, c->titlebar);
    }

    if (DEF_RESIZEBAR) {
      /* Filled rectangle, in which is the same colour as the background */
      XFillRectangle(dpy, c->parent, bkfill_gc, 0, c->height + theight(c),
        c->width, rheight(c));
      /* Resizebar top line */
      draw_line3d(c, 0, c->height + theight(c),
        c->width, c->height + theight(c));
      /* Resizebar right side line */
      draw_line3d(c, c->width - theight(c)*2, c->height + theight(c),
        c->width - theight(c)*2, c->height + theight(c) + rheight(c));
    }
  }
}

void change_gravity(Client *c, int multiplier) {
  int dy = 0;
  int gravity = (c->size->flags & PWinGravity) ?
    c->size->win_gravity : NorthWestGravity;

  switch (gravity) {
    case NorthWestGravity:
    case NorthEastGravity:
    case NorthGravity: dy = theight(c); break;
    case CenterGravity: dy = theight(c)/2; break;
  }

  c->y += multiplier * dy;
}

#ifdef SHAPE
void set_shape(Client *c) {
  int n, order;
  XRectangle *rect;

  rect = XShapeGetRectangles(dpy, c->window, ShapeBounding, &n, &order);
  if (n > 1) XShapeCombineShape(dpy,
    c->parent, ShapeBounding, 0, theight(c),
    c->window, ShapeBounding, ShapeSet);
  XFree(rect);
}
#endif

void send_wm_delete(Client *c) {
  int i, n, found = 0;
  Atom *protocols;

  if (c) {
    if (XGetWMProtocols(dpy, c->window, &protocols, &n))
      for (i=0; i<n; i++) if (protocols[i] == xa_wm_delete) found++;
    if (found) send_xmessage(c->window, xa_wm_protos, xa_wm_delete);
    else XKillClient(dpy, c->window);
  }
  current = NULL;
}

int send_xmessage(Window w, Atom a, long x) {
  XEvent ev;

  ev.type = ClientMessage;
  ev.xclient.window = w;
  ev.xclient.message_type = a;
  ev.xclient.format = 32;
  ev.xclient.data.l[0] = x;
  ev.xclient.data.l[1] = CurrentTime;

  return XSendEvent(dpy, w, False, NoEventMask, &ev);
}
