/* alloywm - (Somewhat) Minimalist Window Manager for the X Window system
 * Copyright (C) 2000, Kensuke Otake <kensuke@phreaker.net>
 * see README for license and other mindless garbage. */

#include "alloywm.h"
#include <stdlib.h>
#include <X11/Xatom.h>

void handle_key_event(XKeyEvent *e) {
  Client *c = find_client(e->window, 1);

  if (!c)
    c = current;
  switch (XKeycodeToKeysym(dpy,e->keycode,0)) {
    case XK_h: case XK_j: case XK_k: case XK_l:
    case XK_y: case XK_u: case XK_b: case XK_n:
      if (c) { XPutBackEvent(dpy, (XEvent *)e); move(c); }
      break;
    case XK_Return:
      spawn(opt_term ? opt_term : DEF_TERM); break;
    case XK_Escape:
      send_wm_delete(c); break;
    case XK_Tab:
      if (c && wm_state(c) == NormalState) next(c);
      break;
    case XK_space:
      if (!c) return;
      if (!c->trans) rollup(c);
      break;
    case XK_period:
      if (!c) return; else hide(c);
      break;
    case XK_Insert: case XK_KP_Insert:
      if (c) lower_client(c);
      break;
  }
}

void handle_button_event(XButtonEvent *e) {
  Client *c = find_client(e->window, 1);
  
  /* A happy shitload of boolean events are below. */
  int titlebar = 0;
  int window = 0;
  int resizebox = 0, closebox = 0, minimbox = 0, maximbox = 0, iconifybox = 0;

  /*if (e->window == root) {
    switch (e->button) {
      case Button1: system(opt_new1 ? opt_new1 : DEF_NEW1 "&"); break;
      case Button3: system(opt_new2 ? opt_new2 : DEF_NEW2 "&"); break;
    }
  } else*/ if (e->window == c->window) {
    switch (e->button) {
      case Button1:
        XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
        raise_client(c);
        break;
      case Button3:
        lower_client(c);
        break;
    }
  } else if (c) {
    closebox = (awidget_clicked(c->titlebar.closebox));
    minimbox = (c->width >= theight(c)*2) && awidget_clicked(c->titlebar.minimbox);
    maximbox = (c->width >= theight(c)*4) && awidget_clicked(c->titlebar.maximbox);
    iconifybox = (c->width >= theight(c)*3) && awidget_clicked(c->titlebar.iconfbox);
    resizebox = (DEF_RESIZEBAR) &&
      (e->x >= c->width - theight(c)*2 + opt_bw/2 &&
       e->x <= c->width - theight(c)*2 - opt_bw/2 + theight(c)*2 &&
       e->y >= c->height + theight(c) + opt_bw/2 &&
       e->y <= c->height + theight(c) - opt_bw/2 + rheight(c));
    window = (e->y >= theight(c) && e->y <= c->height + theight(c));
    titlebar = (e->x >= theight(c)*2 && \
                e->x <= c->width - theight(c)*2 && \
                e->y >= 0 && \
                e->y <= theight(c));
    
    switch (e->button) {
      case Button1:
        if (maximbox) {
          maximise(c);
          raise_client(c);
          break;
        } else if (closebox) {
          send_wm_delete(c);
          break;
        } else if (minimbox) {
          rollup(c);
          raise_client(c);
          break;
        } else if (iconifybox) {
          hide(c);
          lower_client(c);
        } else if (resizebox && DEF_RESIZEBAR) {
          resizebox ? resize(c) : move(c);
          break;
        } else if (titlebar || window) {
           raise_client(c);
           move(c);
        }; break;
      case Button2:
        lower_client(c);
      case Button3: break;
    }
  }
}

void handle_configure_request(XConfigureRequestEvent *e) {
  Client *c = find_client(e->window, 0);
  XWindowChanges wc;

  if (c) {
    ungravitate(c);
    if (e->value_mask & CWX) c->x = e->x;
    if (e->value_mask & CWY) c->y = e->y;
    if (e->value_mask & CWWidth) c->width = e->width;
    if (e->value_mask & CWHeight) c->height = e->height;
    gravitate(c);
    
    wc.x = c->x;
    wc.y = c->y - theight(c);
    wc.width = c->width;

    wc.height = c->height + theight(c) + rheight(c);
    wc.border_width = opt_bw;
    wc.sibling = e->above;
    wc.stack_mode = e->detail;
    XConfigureWindow(dpy, c->parent, e->value_mask, &wc);
    send_config(c);
  }

  wc.x = c ? 0 : e->x;
  wc.y = c ? theight(c) : e->y;
  wc.width = e->width;
  wc.height = e->height;
  wc.sibling = e->above;
  wc.stack_mode = e->detail;
  XConfigureWindow(dpy, e->window, e->value_mask, &wc);
}

void handle_map_request(XMapRequestEvent *e) {
  Client *c = find_client(e->window, 0);

  if (c) {
    XMapWindow(dpy, c->window);
    XMapRaised(dpy, c->parent);
    set_wm_state(c, NormalState);
  } else
    make_new_client(e->window);
}

void handle_unmap_event(XUnmapEvent *e) {
  Client *c = find_client(e->window, 0);

  if (c) {
    if (c->ignore_unmap) c->ignore_unmap--;
    else remove_client(c, 0);
  }
}

void handle_client_message(XClientMessageEvent *e) {
  Client *c = find_client(e->window, 0);

  if (c && e->message_type == xa_wm_change_state &&
    e->format == 32 && e->data.l[0] == IconicState) hide(c);
}

void handle_colormap_change(XColormapEvent *e) {
  Client *c = find_client(e->window, 0);

  if (c && e->new) {
    c->cmap = e->colormap;
    XInstallColormap(dpy, c->cmap);
  }
}

void handle_property_change(XPropertyEvent *e) {
  Client *c = find_client(e->window, 0);
  long dummy;

  if (c) {
    switch (e->atom) {
      case XA_WM_NAME:
        if (c->name) XFree(c->name);
        XFetchName(dpy, c->window, &c->name);
        XClearWindow(dpy, c->parent);
        redraw(c);
        break;
      case XA_WM_TRANSIENT_FOR:
        XGetTransientForHint(dpy, c->window, &c->trans);
        break;
      case XA_WM_NORMAL_HINTS:
        XGetWMNormalHints(dpy, c->window, c->size, &dummy);
    }
  }
}

void handle_enter_event(XCrossingEvent *e) {
  Client *c = find_client(e->window, 0);

  if (c) {
    current = c;
    XGrabButton(dpy, AnyButton, Mod1Mask, c->parent, False,
      ButtonMask, GrabModeAsync, GrabModeAsync, None, None);
    if (MOUSEFOCUS || !CLICKFOCUS) {
      XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
      XRaiseWindow(dpy, c->parent);
    } else if (SLOPPYFOCUS) {
      XSetInputFocus(dpy, c->window, RevertToPointerRoot, CurrentTime);
    }
  }
}

void handle_expose_event(XExposeEvent *e) {
  Client *c = find_client(e->window, 1);

  if (c && e->count == 0) {
    redraw(c);
  }
}

void handle_reparent_event(XReparentEvent *e) {
  Client *c = find_client(e->window, 0);
  Client *dest = find_client(e->parent, 1);

  if (c && !dest) remove_client(c, 0);
}

#ifdef SHAPE
void handle_shape_change(XShapeEvent *e) {
  Client *c = find_client(e->window, 0);
  
  if (c) set_shape(c);
}
#endif
