/* alloywm - (Somewhat) Minimalist Window Manager for the X Window system
 * Copyright (c) 2002, Kensuke Otake <kensuke@phreaker.net>
 * see README for license and other mindless garbage. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <X11/cursorfont.h>
#include "alloywm.h"

Display    *dpy;
int    screen;
Window    root;
GC    invert_gc;
GC    string_gc;
GC    border_gc;
GC    threeD_gc;
GC    bkfill_gc;
XFontStruct  *font;
int    iheight;
XColor    fg, bg, bc, bc2;
Client    *head_client;
Client    *current = NULL;
Atom    xa_wm_state;
Atom    xa_wm_change_state;
Atom    xa_wm_protos;
Atom    xa_wm_delete;
Atom    xa_wm_cmapwins;
Cursor    move_curs;
Cursor    resize_curs;
char    *opt_term = DEF_TERM;
char    *opt_display = "";
char    *opt_font = DEF_FONT;
char    *opt_fg = DEF_FG;
char    *opt_bg = DEF_BG;
char    *opt_bc = DEF_BC;
char    *opt_bc2 = DEF_BC2;
char    *opt_new1, *opt_new2;
int    opt_bw = DEF_BW;
#ifdef SHAPE
Bool    shape;
int    shape_event;
#endif

void usage() {
  printf("Usage: %s [-h] [-v] [-d <display>]\n", APPNAME);
}

void version() {
  printf("%s version %s ", APPNAME, APPVERS);
  printf("(c) 2002, Hitokiro Otake.\n");
}

int main(int argc, char **argv) {
  int i, x;
  char prev_flag = '\0';

  for (i=1; i<argc; i++) {
    if (argv[i][0] == '-') {
      for (x=1; argv[i][x]; x++) {
        prev_flag = argv[i][x];
        switch (argv[i][x]) {
          case 'h': {
            usage();
            exit(0);
            break;
          }
          case 'v': {
            version();
            exit(0);
            break;
          }
          case 'd': {
            break;
          }
          default: {
            fprintf(stderr, "Invalid flag '%c', see %s -h.\n", argv[i][x], APPNAME);
            exit(1);
            break;
          }
        }
      }
    } else {
      if (prev_flag == 'd') {
        prev_flag = '\0';
        opt_display = argv[i];
        continue;
      }
    }
  }
  
  setup_display();
  scan_windows();
  do_event_loop();
  exit(1); /* just another brick in the -Wall... */
}

void setup_display() {
  XGCValues gv, gv2;
  XSetWindowAttributes attr;
  XColor dummy;
  int *i;
  int keys_to_grab[] = {XK_Return, XK_Escape,
    XK_Left, XK_Up, XK_Down, XK_Right,
    XK_y, XK_u, XK_b, XK_n, XK_h, XK_j, XK_k, XK_l,
    XK_space, XK_period, XK_Insert, XK_KP_Insert, 0};
#ifdef SHAPE
  int dummy2;
#endif

  XSetErrorHandler(handle_xerror);
  signal(SIGTERM, quit_nicely);
  signal(SIGINT, quit_nicely);
  signal(SIGHUP, quit_nicely);
  signal(SIGCHLD, SIG_IGN);

  dpy = XOpenDisplay(opt_display);

  if (!dpy) { 
    err("can't open display %s", opt_display);
    exit(1);
  }

  screen = DefaultScreen(dpy);
  root = RootWindow(dpy, screen);

  xa_wm_state = XInternAtom(dpy, "WM_STATE", False);
  xa_wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
  xa_wm_protos = XInternAtom(dpy, "WM_PROTOCOLS", False);
  xa_wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
  xa_wm_cmapwins = XInternAtom(dpy, "WM_COLORMAP_WINDOWS", False);

  XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_fg, &fg, &dummy);
  XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bg, &bg, &dummy);
  XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bc, &bc, &dummy);
  XAllocNamedColor(dpy, DefaultColormap(dpy, screen), opt_bc2, &bc2, &dummy);
  
  font = XLoadQueryFont(dpy, opt_font);
  if (!font) font = XLoadQueryFont(dpy, DEF_FONT);
  iheight = font->ascent + font->descent + 2*SPACE;

#ifdef SHAPE
  shape = XShapeQueryExtension(dpy, &shape_event, &dummy2);
#endif
  move_curs = XCreateFontCursor(dpy, XC_fleur);
  resize_curs = XCreateFontCursor(dpy, XC_plus);

  gv.function = GXcopy;
  gv.foreground = fg.pixel;
  gv.font = font->fid;
  string_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCFont, &gv);

  gv.function = GXcopy;
  gv.foreground = bg.pixel;
  bkfill_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCLineWidth, &gv);

  gv2.function = GXcopy;
  gv2.foreground = bc2.pixel;
  gv2.line_width = opt_bw;
  threeD_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCLineWidth, &gv2);
  
  gv.foreground = bc.pixel;
  gv.line_width = opt_bw;
  border_gc = XCreateGC(dpy, root, GCFunction | GCForeground | GCLineWidth, &gv);
  
  gv.function = GXinvert;
  gv.subwindow_mode = IncludeInferiors;
  invert_gc = XCreateGC(dpy, root, GCFunction | GCSubwindowMode | GCLineWidth | GCFont, &gv);

  attr.event_mask = ChildMask | ColormapChangeMask | ButtonMask | PropertyChangeMask;
  XChangeWindowAttributes(dpy, root, CWEventMask, &attr);
  for (i = keys_to_grab; *i; i++) {
    XGrabKey(dpy, XKeysymToKeycode(dpy, *i), ControlMask | Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
  }
  XGrabKey(dpy, XKeysymToKeycode(dpy, XK_Tab), Mod1Mask, root, True, GrabModeAsync, GrabModeAsync);
}

void scan_windows() {
  unsigned int i, nwins;
  Window dw1, dw2, *wins;
  XWindowAttributes attr;

  XQueryTree(dpy, root, &dw1, &dw2, &wins, &nwins);
  for (i = 0; i < nwins; i++) {
    XGetWindowAttributes(dpy, wins[i], &attr);
    if (!attr.override_redirect && attr.map_state == IsViewable)
      make_new_client(wins[i]);
  }
  XFree(wins);
}
