/* alloywm - (Somewhat) Minimalist Window Manager for the X Window system
 * Copyright (c) 2000, Kensuke Otake <kensuke@phreaker.net>
 * see README for license and other mindless garbage. */

#include <stdio.h>
#include "alloywm.h"

void draw_outline(Client *c) {
  char buf[40];

  sprintf(buf, "%dx%d+%d+%d", c->width, c->height, c->x, c->y);
  if (c && c->shaded) {
    XDrawRectangle(dpy, root, invert_gc,
      c->x, c->y - theight(c),
      c->width + opt_bw, theight(c));
    XDrawString(dpy, root, invert_gc,
      c->x + opt_bw,
      c->y + opt_bw/2 - SPACE,
      buf, strlen(buf));
  } else if (c && !c->shaded) {
    /* if (c->trans) XDrawLine(dpy, root, invert_gc,
        c->x, c->y,
        c->x + c->width, c->y); */
    XDrawRectangle(dpy, root, invert_gc,
      c->x, c->y - theight(c),
      c->width + opt_bw, c->height + theight(c) + rheight(c) + opt_bw);
    if (!c->trans) XDrawLine(dpy, root, invert_gc, c->x + opt_bw, c->y,
      c->x + c->width + opt_bw, c->y);
    if (DEF_RESIZEBAR && !c->trans) {
      XDrawLine(dpy, root, invert_gc, c->x + opt_bw, c->y + c->height + opt_bw,
        c->x + c->width + opt_bw, c->y + c->height + opt_bw);
    }
    XDrawString(dpy, root, invert_gc,
      c->x + c->width - XTextWidth(font, buf, strlen(buf)) - SPACE,
      c->y + c->height - SPACE,
      buf, strlen(buf));
  }
}

void get_mouse_position(int *x, int *y) {
  Window dw1, dw2;
  int t1, t2;
  unsigned int t3;

  XQueryPointer(dpy, root, &dw1, &dw2, x, y, &t1, &t2, &t3);
}

void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
  int basex, basey;

  c->width = abs(x1 - x2) - opt_bw;
  c->height = abs(y1 - y2) - opt_bw;

  if (c->size->flags & PResizeInc) {
    basex = (c->size->flags & PBaseSize) ? c->size->base_width :
      (c->size->flags & PMinSize) ? c->size->min_width : 0;
    basey = (c->size->flags & PBaseSize) ? c->size->base_height :
      (c->size->flags & PMinSize) ? c->size->min_height : 0;
    c->width -= (c->width - basex) % c->size->width_inc;
    c->height -= (c->height - basey) % c->size->height_inc;
  }

  if (c->size->flags & PMinSize) {
    if (c->width < c->size->min_width) c->width = c->size->min_width;
    if (c->height < c->size->min_height) c->height = c->size->min_height;
  }

  if (c->size->flags & PMaxSize) {
    if (c->width > c->size->max_width) c->width = c->size->max_width;
    if (c->height > c->size->max_height) c->height = c->size->max_height;
  }

  c->x = (x1 <= x2) ? x1 : x1 - c->width;
  c->y = (y1 <= y2) ? y1 : y1 - c->height;
}

void sweep(Client *c) {
  XEvent ev;
  int old_cx = c->x;
  int old_cy = c->y;

  if (!grab(root, MouseMask, resize_curs)) return;
  XGrabServer(dpy);

  draw_outline(c);
  for (;;) {
    XMaskEvent(dpy, MouseMask, &ev);
    switch (ev.type) {
      case MotionNotify:
        draw_outline(c); /* clear */
        recalculate_sweep(c, old_cx, old_cy, ev.xmotion.x, ev.xmotion.y);
        draw_outline(c);
        break;
      case ButtonRelease:
        draw_outline(c); /* clear */
        ungrab();
        return;
    }
  }
}

void drag(Client *c) {
  XEvent ev;
  int x1, y1;
  int old_cx = c->x;
  int old_cy = c->y;

  if (!grab(root, MouseMask, move_curs)) return;
  XGrabServer(dpy);
  get_mouse_position(&x1, &y1);

  if (!DEF_OPAQUE) draw_outline(c);
  
  for (;;) {
    XMaskEvent(dpy, ButtonMotionMask | ButtonPressMask | ButtonRelease | KeyReleaseMask | KeyPressMask | MouseMask, &ev);
    switch (ev.type) {
      case MotionNotify:
        StashEventTime(&ev);
        if (!DEF_OPAQUE) draw_outline(c); /* clear */
        c->x = old_cx + (ev.xmotion.x - x1);
        c->y = old_cy + (ev.xmotion.y - y1);

        /* This is where alloywm decides whether or
         * draw the window as it is moved. */
        if (DEF_OPAQUE) XMoveWindow(dpy, c->parent, c->x, c->y - theight(c));
        if (!DEF_OPAQUE) draw_outline(c);
        break;
      case ButtonRelease:
        if (!DEF_OPAQUE) draw_outline(c); /* clear */
        ungrab();
        return;
      case KeyPress:
        if (!DEF_OPAQUE) draw_outline(c); /* clear */
        switch(XKeycodeToKeysym(dpy, ev.xkey.keycode, 0)) {
          /* Capture key events to change window states. */
          case XK_space:
            if (!c->trans && c) rollup(c);
          case XK_period:
            if (!c->trans) hide(c); break;

          /* Capture key events to move windows. */
          case XK_Left:  case XK_h: c->x -= 16; break;
          case XK_Down:  case XK_j: c->y += 16; break;
          case XK_Up:    case XK_k: c->y -= 16; break;
          case XK_Right: case XK_l: c->x += 16; break;
          case XK_y: c->x = c->border; c->y = c->border + theight(c); break;
          case XK_u: c->x = DisplayWidth(dpy, screen) - c->width - c->border; c->y = c->border + theight(c); break;
          case XK_b: c->x = c->border; c->y = DisplayHeight(dpy, screen) - c->height - c->border; break;
          case XK_n: c->x = DisplayWidth(dpy, screen) - c->width - c->border; c->y = DisplayHeight(dpy, screen) - c->height - c->border; break;
        }
        ungrab();
        setmouse(c->window, c->width - c->width/2, c->height - c->height/2);
        return;
        /* draw_outline(c); */
        break;
      case ButtonPress:
        XAllowEvents(dpy, ReplayPointer, CurrentTime);
        break;
      default:
        break;

    }
  }
}

void move(Client *c) {
  if (c) {
    drag(c);
    XMoveWindow(dpy, c->parent, c->x, c->y - theight(c));
    send_config(c);
  }
}

void resize(Client *c) {
  if (c) {
    sweep(c);
    XMoveResizeWindow(dpy, c->parent,
      c->x, c->y - theight(c), c->width, c->height + theight(c) + rheight(c));
    XMoveResizeWindow(dpy, c->window,
      0, theight(c), c->width, c->height);
    send_config(c);
  }
}

void hide(Client *c) {
  if (c) {
    c->ignore_unmap++;
    XUnmapWindow(dpy, c->parent);
    XUnmapWindow(dpy, c->window);
    set_wm_state(c, IconicState);
  } 
}

void unhide(Client *c) {
  if (c) {
    XMapWindow(dpy, c->window);
    XMapRaised(dpy, c->parent);
    set_wm_state(c, NormalState);
  }
}

void next(Client *c) {
  Client *newc;

  if (c) {
    if (c->next) {
      newc = c->next;
    } else {
      newc = head_client;
    }
    
    if (newc->shaded) rollup(newc);
    if (wm_state(newc) == IconicState) unhide(newc);

    if (c->trans) setmouse(newc->window, newc->width - SPACE*4, newc->height - SPACE*4);
    else
    setmouse(newc->window, newc->width - theight(c), newc->height - theight(c));
    raise_client(newc);
  }
}
