/* alloywm - (Somewhat) Minimalist Window Manager for the X Window system
 * Copyright (c) 2000, Kensuke Otake <kensuke@phreaker.net>
 * see README for license and other mindless garbage. */

#include "alloywm.h"
#include <stdlib.h>

/* A lovely bunch of titlebar widgets for your drawing disgression. */

/* Draw a line in "3D", a mere extension to draw_line3d(c, ). */
void draw_line3d(Client *c, int x1, int y1, int x2, int y2) {
  XDrawLine(dpy, c->parent, threeD_gc,
    x1 + 1, y1 + 1, x2 + 1, y2 + 1);  
  XDrawLine(dpy, c->parent, border_gc,
    x1, y1, x2, y2);
}

void draw_inv_line3d(Client *c, int x1, int y1, int x2, int y2) {
  XDrawLine(dpy, c->parent, border_gc,
    x1 + 1, y1 + 1, x2 + 1, y2 + 1);
  XDrawLine(dpy, c->parent, threeD_gc,
    x1, y1, x2, y2);
}

/* Position the widgets in their respective spots. */
void position_awidget(Client *c, AWidget *widget) {
  switch (widget->type) {
    case AWidget_Maximise:
      widget->x = AWidget_Maximise_X_Position();
      widget->y = -1;
      break;
    case AWidget_Minimise:
      widget->x = AWidget_Minimise_X_Position();
      widget->y = -1;
      break;
    case AWidget_Close:
      widget->x = AWidget_Close_X_Position();
      widget->y = -1;
      break;
    case AWidget_Iconify:
      widget->x = AWidget_Iconify_X_Position();
      widget->y = -1;
      break;
  }
}

/* Create the basic widget */
AWidget make_awidget(int AWidget_Type, int x, int y, int w, int h) {
  AWidget widget;
  widget.x = x;
  widget.y = y;
  widget.width = w + opt_bw*2 + opt_bw;
  widget.height = w + opt_bw*2 + opt_bw;
  widget.type = AWidget_Type;
  widget.state = AState_Normal;

  return widget;
}

/* Redraw a widget of the AWidget datatype */
void redraw_awidget(Client *c, AWidget widget) {
  if (widget.type == AWidget_Maximise)
    draw_maximisebox(c, widget.x, widget.y);
  if (widget.type == AWidget_Iconify)
    draw_iconifybox(c, widget.x, widget.y);
  if (widget.type == AWidget_Minimise)
    draw_minimisebox(c, widget.x, widget.y);
  if (widget.type == AWidget_Close)
    draw_closebox(c, widget.x, widget.y);
  if (widget.type == AWidget_Resize)
    draw_resizebox(c, widget.x, widget.y);
  if (widget.type == AWidget_Menu)
    draw_menubox(c, widget.x, widget.y);
  if (widget.type == AWidget_Empty)
    draw_empty_widget(c, widget.x, widget.y);
}

void redraw_atitlebar(Client *c, ATitlebar titlebar) {
  if (!c) return;

  /* Draw a big-ass square which is the size of the titlebar, so programs
     that set the backing plane of the parent window as the root window
     (like aterm) can't draw in the titlebar as well. */
  XFillRectangle(dpy, c->parent, bkfill_gc, 0, 0, c->width, theight(c));
  
  /* Draw title if window is not a transient, and if the client width
     is greater than or equal to the width of five widgets */

  if (!c->trans && c->width >= theight(c)*4) {
    draw_string(theight(c)*2 + opt_bw/2 + opt_bw + SPACE,
      SPACE + font->ascent, c->name);
  }

  /* Bottom line */
  draw_line(0, theight(c) - opt_bw + opt_bw/2,
    c->width, theight(c) - opt_bw + opt_bw/2);
  /* Top line */
  draw_line3d(c, -1, -1, c->width, -1);
  /* Left line (In case there are no widgets on the left) */
  draw_line3d(c, -1, -1, -1, theight(c));

  if (c->width >= theight(c)) redraw_awidget(c, titlebar.closebox);
  if (c->width >= theight(c)*2) redraw_awidget(c, titlebar.maximbox);
  if (c->width >= theight(c)*3) redraw_awidget(c, titlebar.minimbox);
  if (c->width >= theight(c)*4) redraw_awidget(c, titlebar.iconfbox);
}

void draw_empty_widget(Client *c, int x, int y) {
  XFillRectangle(dpy, c->parent, bkfill_gc, x, y, theight(c), theight(c));
  
  draw_line(x, y, x, y + theight(c));
  draw_line(x, y + theight(c), x + theight(c), y + theight(c));
  
  draw_line3d(c, x, y-1, x, y-1 + theight(c));
  draw_line3d(c, x, y, x + theight(c), y);
  
  draw_line(x + theight(c), y, x + theight(c), y + theight(c));
  draw_line(x, y, x + theight(c), y);
}

void draw_depressed_empty_widget(Client *c, int x, int y) {
  draw_line(x, y, x, y + theight(c));
  draw_line(x, y + theight(c), x + theight(c), y + theight(c));
  
  draw_inv_line3d(c, x, y, x + theight(c), y);
  draw_inv_line3d(c, x, y-1, x, y-1 + theight(c));

  draw_line(x + theight(c), y, x + theight(c), y + theight(c));
  draw_line(x, y, x + theight(c), y);
}

void draw_depressed_widget(Client *c, AWidget widget) {
  redraw_awidget(c, widget);
  draw_depressed_empty_widget(c, widget.x, widget.y);
}

void draw_closebox(Client *c, int x, int y) {
  int X = x + theight(c)/2;
  int Y = y + theight(c)/2;

  draw_empty_widget(c, x, y);
  
  /* \ Line */
  draw_line(X-theight(c)/4, Y-theight(c)/4, X+theight(c)/4 + 1, Y+theight(c)/4 + 1);
  /* / Line */
  draw_line(X+theight(c)/4+1, Y-theight(c)/4-1, X-theight(c)/4, Y+theight(c)/4);
}

void draw_maximisebox(Client *c, int x, int y) {
  draw_empty_widget(c, x, y);
  
  /* Top line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c)/4);
  /* Bottom line */
  draw_line(x + theight(c)/4, y + theight(c) - theight(c)/4,
    x + theight(c) - theight(c)/4 + opt_bw/2 + opt_bw, y + theight(c) - theight(c)/4);
  /* Left line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c)/4, y + theight(c) - theight(c)/4);
  /* Right line */
  draw_line(x + theight(c) - theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c) - theight(c)/4 + opt_bw/2);

  /* Centre horizontal line */
  draw_line(x + theight(c)/4, y + theight(c)/2,
    x + theight(c) - theight(c)/4 - theight(c)/6, y + theight(c)/2);  
  /* Centre vertical line */
  draw_line(x + theight(c) - theight(c)/4 - theight(c)/6 - opt_bw/2 - opt_bw, y + theight(c)/2,
    x + theight(c) - theight(c)/4 - theight(c)/6 - opt_bw/2 - opt_bw, y + theight(c)/4);
}

void draw_minimisebox(Client *c, int x, int y) {
  draw_empty_widget(c, x, y);
  
  /* Bottom line */
  draw_line(x + theight(c)/4, y + theight(c) - theight(c)/4,
    x + theight(c) - theight(c)/4 + opt_bw/2 + opt_bw, y + theight(c) - theight(c)/4);
  /* Left line */
  draw_line(x + theight(c)/4, y + theight(c)/2 + opt_bw*2,
    x + theight(c)/4, y + theight(c) - theight(c)/4);
  /* Right line */
  draw_line(x + theight(c) - theight(c)/4, y + theight(c)/2 + opt_bw*2,
    x + theight(c) - theight(c)/4, y + theight(c) - theight(c)/4 + opt_bw/2);

  /* Top line */
  draw_line(x + theight(c)/4, y + theight(c)/2 + opt_bw*2,
    x + theight(c) - theight(c)/4, y + theight(c)/2 + opt_bw*2);
}

void draw_menubox(Client *c, int x, int y) {
  int X = x + theight(c)/2;
  int Y = y + theight(c)/2;

  draw_empty_widget(c, x, y);
  
  /* Top line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c)/4);
  /* Bottom line */
  draw_line(x + theight(c)/4, y + theight(c) - theight(c)/4,
    x + theight(c) - theight(c)/4 + opt_bw/2 + opt_bw, y + theight(c) - theight(c)/4);
  /* Left line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c)/4, y + theight(c) - theight(c)/4);
  /* Right line */
  draw_line(x + theight(c) - theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c) - theight(c)/4 + opt_bw/2);

  /* Centre corner-to-corner line */
  draw_line(X+5, Y-5, X-4, Y+4);
}

void draw_resizebox(Client *c, int x, int y) {
  int X = x + theight(c)/2;
  int Y = y + theight(c)/2;

  draw_empty_widget(c, x, y);
  
  draw_line(X+5, Y-5, X-4, Y+4);
  draw_line(X+5, Y-9, X-8, Y+4);
}

void draw_iconifybox(Client *c, int x, int y) {
  draw_empty_widget(c, x, y);
  
  /* Top line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c)/4);
  /* Bottom line */
  draw_line(x + theight(c)/4, y + theight(c) - theight(c)/4,
    x + theight(c) - theight(c)/4 + opt_bw/2 + opt_bw, y + theight(c) - theight(c)/4);
  /* Left line */
  draw_line(x + theight(c)/4, y + theight(c)/4,
    x + theight(c)/4, y + theight(c) - theight(c)/4);
  /* Right line */
  draw_line(x + theight(c) - theight(c)/4, y + theight(c)/4,
    x + theight(c) - theight(c)/4, y + theight(c) - theight(c)/4 + opt_bw/2);

  /* Centre line */
  draw_line(x + theight(c)/4, y + theight(c)/2 + opt_bw*2,
    x + theight(c) - theight(c)/4, y + theight(c)/2 + opt_bw*2);
}

void update_atitlebar(Client *c, ATitlebar *titlebar) {
  /* A minor fetish of saving CPU cycles. */
  if (c) {
    position_awidget(c, &titlebar->maximbox);
    position_awidget(c, &titlebar->minimbox);
    position_awidget(c, &titlebar->closebox);
    position_awidget(c, &titlebar->iconfbox);
  }
}

